<?php

namespace Drupal\drd\Generators;

use Drupal\Core\Extension\ModuleHandlerInterface;
use DrupalCodeGenerator\Command\ModuleGenerator;
use DrupalCodeGenerator\Utils;

/**
 * Action Plugin generator for Drupal Code Generator 2.x.
 *
 * @package Drupal\drd\Generators
 */
class ActionPlugin extends ModuleGenerator {

  /**
   * {@inheritdoc}
   */
  protected string $name = 'drd-action-plugin';

  /**
   * {@inheritdoc}
   */
  protected string $description = 'Generates an action plugin for DRD.';

  /**
   * {@inheritdoc}
   */
  protected string $alias = 'drdap';

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface|null
   */
  protected ?ModuleHandlerInterface $moduleHandler;

  /**
   * ActionPlugin constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface|null $moduleHandler
   *   The containers modules handler.
   * @param string|null $name
   *   The name of the command; passing null means it must be set in
   *   configure().
   */
  public function __construct(ModuleHandlerInterface $moduleHandler = NULL, string $name = NULL) {
    $this->templatePath = __DIR__ . '/templates';
    parent::__construct($name);
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * Get extra vars depending on the selected action plugin type.
   *
   * @param array $vars
   *   Collected vars.
   *
   * @return array
   *   The extra vars.
   */
  private function typeVars(array $vars): array {
    $type_vars = [
      'drd' => [
        'dc_base_class' => 'BaseSystem',
      ],
      'drd_host' => [
        'dc_base_class' => 'BaseHost',
        'drush_callback' => 'hosts',
      ],
      'drd_core' => [
        'dc_base_class' => 'BaseCore',
        'drush_callback' => 'cores',
      ],
      'drd_domain' => [
        'dc_base_class' => 'BaseDomain',
        'drush_callback' => 'domains',
      ],
    ];

    return $type_vars[$vars['type']];
  }

  /**
   * Set a file for rendering.
   *
   * @param string $path
   *   The sub-path within the templates directory.
   * @param string $filename
   *   The destination filename.
   * @param array $vars
   *   Collected vars.
   * @param bool $append
   *   Whether to append (TRUE) or replace (FALSE) the destination file.
   * @param string|null $twigfile
   *   If null, the source filename will be the destination filename with the
   *   appended ".twig" extension, and if the source filename is different, this
   *   should be stated here.
   */
  private function setOrAppendFile(string $path, string $filename, array $vars, bool $append = TRUE, string $twigfile = NULL): void {
    if (!isset($twigfile)) {
      $twigfile = $filename . '.twig';
    }
    $callback = $append ? 'addServicesFile' : 'addFile';
    $this->{$callback}(
      trim($path . '/' . $filename, '/'),
      trim($path . '/' . $twigfile, '/'),
      $vars
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function generate(array &$vars): void {
    $this->collectDefault($vars);

    $validator = [$this, 'validateRequired'];
    $vars['id'] = $this->ask('Action ID', NULL, $validator);
    $vars['label'] = $this->ask('Action Label', NULL, $validator);
    $action_types = [
      'drd',
      'drd_host',
      'drd_core',
      'drd_domain',
    ];
    $vars['type'] = $this->choice('Action Type', \array_combine($action_types, $action_types), 'drd_domain');

    $vars['class'] = Utils::camelize($vars['id']);
    $vars += $this->typeVars($vars);

    $this->setOrAppendFile('', 'drush.services.yml', $vars);
    $this->setOrAppendFile('config/optional', 'system.action.' . $vars['machine_name'] . '_drd_action_' . $vars['id'] . '.yml', $vars, FALSE, 'system.action.drd_action.yml.twig');
    foreach (['6', '7', '8'] as $version) {
      $this->setOrAppendFile('src/Agent/Action/V' . $version, $vars['class'] . '.php', $vars, FALSE, 'Class.php.twig');
    }
    $this->setOrAppendFile('src/Commands', $vars['class'] . 'Commands.php', $vars, FALSE, 'ClassCommands.php.twig');
    $this->setOrAppendFile('src/Plugin/Action', $vars['class'] . '.php', $vars, FALSE, $vars['type'] . '.php.twig');
  }

}
